package simulation;
import java.util.*;

/**
 *  It represents a simulation with a multi-agent system. Several
 * agentes can be registered in the simulation. The simulation
 * is run with a given number of iterations.
 * @author Iván García-Magariño
 */
public class Simulation {
    /** List of all the agents registered for the simulation **/
    private List<Agent> agents;
    /** Creates a simulation with an empty list of agents **/
    public Simulation(){
        agents = new LinkedList<Agent>();
    }
    /** Register an agent in the simulation **/
    public void register(Agent agent){
        agents.add(agent);
    }
    /** Run an iteration of the simulation by calling live
     * method of all agents.
     */
    public void live(){
        Iterator<Agent> it = agents.iterator();
        Agent agent;
        while(it.hasNext()){
            agent = it.next();
            agent.live();
        }
    }
    /** Runs the whole simulation with the left number of iterations 
     * indicated in the parameter.
     */
    public void run (int numIterations){
        for(int i=0;i<numIterations;i++){
            live();
        }
    }
    /** Returns the number of the agents in the simulation **/
    public int getSize(){
        return agents.size();
    }
    
}
