/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package sociogram;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import javax.swing.WindowConstants;

/**
 * This class represents the GUI of a sociagram to show its evolution through
 * the simulation.
 * @author Iván García-Magariño
 */
public class SociogramGUI extends javax.swing.JFrame {
    /** Width in pixels of the recommended size of the sociogram **/
    public static double WIDTH=500;
    /** Height in pixels of the recommended size of the sociogram **/
    public static double HEIGHT=600;
    /** Width in pixels of the recommended size of the sociogram window **/
    public static int WIDTH_WINDOW=770;
    /** Height in pixels of the recommended size of the sociogram window **/
    public static int HEIGHT_WINDOW=550;   
    /** Coordinates of the center **/
    public static double CENTER_X=WIDTH/2;
    public static double CENTER_Y=HEIGHT/2; 
    /** Radius in pixels of the sociogram */
    public static double RADIUS=200;
    /** Size of each agent **/
    public static double AGENT_SIZE=6;
    /** Thickness of the relationship of 100% **/
    public static double THICKNESS=1;
    /** Features of the Legend **/
    public static double MARGIN_LEFT_LEGEND=20;
    public static double MARGIN_TOP_LEGEND=40;
    public static double FONT_SIZE_LEGEND=14;
    public static double FONT_SIZE_ITERATION=20;
    public static double FONT_SIZE_HEADER=20;
    public static double WIDTH_RECTANGLE_LENGED=30;
    public static double HEIGHT_RECTANGLE_LEGEND=20;
    public static double INTERLINE_LEGEND=27;
    public static double SPACE=5;
    /** Features of the Arrow **/
    public static double ARROWHEAD_RATIO=0.8; // Where to place the arrowhead in interval [0,1]
    public static double ARROWHEAD_ANGLE=0.5; // Angle in radian to obtain the lines of the arrowhead
    public static double ARROWHEAD_DISTANCE=25; // How long arrowhead lines are in pixels
    /** Background Color **/
    public static Color BACKGROUND_COLOR=Color.WHITE;
    /** Color of the selection relationships among agents **/
    public static Color SELECTION_COLOR=Color.BLACK;
    /** Color of the rejection relationships among agents **/
    public static Color REJECTION_COLOR=Color.RED;
    /** Colors of each agent type **/
    public static Color colors[]={    
                        Color.YELLOW, // QUITE
                        Color.GREEN,  // PARTICIPANT
                        Color.PINK,  // TANGENT
                        Color.BLUE,  // JOKER
                        Color.MAGENTA,  // OBSTRUCTIVE
                        Color.CYAN};  // OCCASIONAL
    
    /**
     * Creates new form SociogramGUI
     */
    public SociogramGUI() {
        initComponents();
        // User
        this.setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
        this.setSize(new Dimension(WIDTH_WINDOW,HEIGHT_WINDOW));
    }
    
    /** 
     * Paints the sociogram.
     */
    @Override
    public void paint(Graphics g){
        // Set the background color
        g.setColor(this.BACKGROUND_COLOR);
        g.fillRect(0, 0, this.getWidth(), this.getHeight());
        // Represents graphically the agents
        int size = Blackboard.getInstance().getSociogram().getSize();
        IndividualAgent agents[]=Blackboard.getInstance().getSociogram().getAgents();
        double angle=0;
        double inc=2*Math.PI/size;
        double x,y;
        for(int i=0;i<size;i++){
            // Calculate the position of each agent
            x=CENTER_X + RADIUS*Math.cos(angle);
            y=CENTER_Y + RADIUS*Math.sin(angle);
            // Establish the graphical position of the agent
            agents[i].setX(x);
            agents[i].setY(y);
            // Select the color according to the agent type
            g.setColor(colors[IndividualAgent.intValue(agents[i].getType())]);
            // Paint the agent
            g.fillOval((int)(x-AGENT_SIZE/2),(int)(y-AGENT_SIZE/2),
                    (int)AGENT_SIZE,(int)AGENT_SIZE);
            // Increment the angle
            angle=angle+inc;
        }
        // Represents graphically the relations among agents
        Graphics2D g2 = (Graphics2D) g;
        double matrix[][]=Blackboard.getInstance().getSociogram().getMatrix();
        double x1,y1,x2,y2;
        for(int i=0;i<size;i++){
            for(int j=0;j<size;j++){
               x1=agents[i].getX();
               y1=agents[i].getY();
               x2=agents[j].getX();
               y2=agents[j].getY();
               // Threshold for representing relationship
               double relation =matrix[i][j];
               if(relation!=0){
                   // Change the thickness of the relation
                   int thick= (int) (THICKNESS*Math.abs(relation));
                   g2.setStroke(new BasicStroke(thick));
                   // Change the color according to the sign
                   if(relation>0){
                       g2.setColor(this.SELECTION_COLOR);
                   }else if (relation<0){
                       g2.setColor(this.REJECTION_COLOR);
                   }
                   // Draw the Arrow
                   paintArrow(g2, (int)x1,(int)y1,(int)x2,(int)y2);
                   
               }
               
            }
        }
        // Represents the legend of color types
        paintLegend(g);
    }
    
    /** Paints an arrow from (x1,y1) to (x2,y2)*/
    private void paintArrow(Graphics2D g2, int x1, int y1, int x2, int y2){
        // Paints the main line of the arrow
        g2.drawLine(x1,y1,x2,y2);
        // Calculates the point where to place the arrowhead given a ratio
        double xhead, yhead;
        xhead = x1+ARROWHEAD_RATIO*(x2-x1);
        yhead = y1+ARROWHEAD_RATIO*(y2-y1);
        // Calculates the angle of the main line of the arrow
        double angleMain;
        angleMain=Math.atan2(y2-y1,x2-x1);
        // calculates the angles of arrowheads
        double angleHead1, angleHead2, angleAux;
        angleAux=angleMain+Math.PI;
        angleHead1=angleAux+ARROWHEAD_ANGLE;
        angleHead2=angleAux-ARROWHEAD_ANGLE;
        // Paints the arrowhead lineas
        double xhead1, yhead1, xhead2, yhead2;
        xhead1=xhead+Math.cos(angleHead1)*ARROWHEAD_DISTANCE;
        yhead1=yhead+Math.sin(angleHead1)*ARROWHEAD_DISTANCE;
        g2.drawLine((int)xhead,(int)yhead,(int)xhead1,(int)yhead1);
        xhead2=xhead+Math.cos(angleHead2)*ARROWHEAD_DISTANCE;
        yhead2=yhead+Math.sin(angleHead2)*ARROWHEAD_DISTANCE;
        g2.drawLine((int)xhead,(int)yhead,(int)xhead2,(int)yhead2);
        
    }
    
    /** Paints the Legend. This includes the types of agents, the types of
     relationships, and the number of the iteration and sociometrics. **/
    private void paintLegend(Graphics g){
        double x, y;
        x=this.WIDTH+this.MARGIN_LEFT_LEGEND;
        y=this.MARGIN_TOP_LEGEND;
        // Paints the colors of agent types
        g.setColor(Color.BLACK);
        g.setFont(new Font("News Roman", Font.PLAIN, (int)this.FONT_SIZE_HEADER));
        g.drawString("Colors of the agent types:",(int)x, (int)(y+HEIGHT_RECTANGLE_LEGEND));
        g.setFont(new Font("News Roman", Font.PLAIN, (int)this.FONT_SIZE_LEGEND));
        y+=this.INTERLINE_LEGEND;
        for(AgentType type: AgentType.values()){
            g.setColor(colors[IndividualAgent.intValue(type)]);
            g.fillRect((int)x, (int)y, (int) this.WIDTH_RECTANGLE_LENGED, 
                    (int)this.HEIGHT_RECTANGLE_LEGEND);
            g.setColor(Color.BLACK);
            g.drawString(""+type,
                    (int)(x+WIDTH_RECTANGLE_LENGED+SPACE), 
                    (int)(y+HEIGHT_RECTANGLE_LEGEND));
            y+=this.INTERLINE_LEGEND;
        }
        // Paints a blank line
        y+=this.INTERLINE_LEGEND;
        // Paints the relationship coloros
        g.setFont(new Font("News Roman", Font.PLAIN, (int)this.FONT_SIZE_HEADER));
        g.drawString("Colors of the relationships:",(int)x, (int)(y+THICKNESS));
        g.setFont(new Font("News Roman", Font.PLAIN, (int)this.FONT_SIZE_LEGEND));
        y+=this.INTERLINE_LEGEND;
        g.setColor(SELECTION_COLOR);
        g.fillRect((int)x, (int)y, (int) this.WIDTH_RECTANGLE_LENGED, 
                    (int) THICKNESS);
        g.setColor(Color.BLACK);
        g.drawString("Selection",
                    (int)(x+WIDTH_RECTANGLE_LENGED+SPACE), 
                    (int)(y+THICKNESS));
        y+=this.INTERLINE_LEGEND;
        g.setColor(REJECTION_COLOR);
        g.fillRect((int)x, (int)y, (int) this.WIDTH_RECTANGLE_LENGED, 
                    (int) THICKNESS);
        g.setColor(Color.BLACK);
        g.drawString("Rejection",
                    (int)(x+WIDTH_RECTANGLE_LENGED+SPACE), 
                    (int)(y+THICKNESS));
        y+=this.INTERLINE_LEGEND;
        // Paints a blank line 
        y+=this.INTERLINE_LEGEND;
        // Paints the iteration and sociometrics
        paintMetrics(g, x, y);
        
        
    }
    
    /** Paints the the sociometrics alongside the number of the iteration 
     @param g Graphics of the frame
     @param x horizontal pixel where to start writing the first line. 
     @param y verticla pixel where to start writing the first line. 
     **/
    private void paintMetrics(Graphics g, double x, double y){
        // Paints the title
        g.setFont(new Font("News Roman", Font.PLAIN, (int)this.FONT_SIZE_HEADER));
        g.drawString("Iteration and Sociometrics ", (int)x, (int) y);
        y+=this.INTERLINE_LEGEND;
        // Paints the iterations
        g.setFont(new Font("News Roman", Font.PLAIN, (int)this.FONT_SIZE_LEGEND));
        //int iteration=Blackboard.getInstance().getSimulation().getIteration();
        //g.drawString("Iteration: "+iteration, (int)x, (int) y);
        //y+=this.INTERLINE_LEGEND;
        //  Paints the metrics
        double iag, icg, idg, iig;
        iag=Blackboard.getInstance().getSociogram().measureIAg();
        icg=Blackboard.getInstance().getSociogram().measureICg();
        idg=Blackboard.getInstance().getSociogram().measureIDg();
        iig=Blackboard.getInstance().getSociogram().measureIIg();
        g.drawString("Cohesion:          IAg="+Sociogram.METRIC_FORMAT.format(iag), (int)x, (int) y);
        y+=this.INTERLINE_LEGEND;
        g.drawString("Coherence:       ICg="+Sociogram.METRIC_FORMAT.format(icg), (int)x, (int) y);
        y+=this.INTERLINE_LEGEND;
        g.drawString("Dissociation:      IDg="+Sociogram.METRIC_FORMAT.format(idg), (int)x, (int) y);
        y+=this.INTERLINE_LEGEND;
        g.drawString("Group Intensity: IIg="+Sociogram.METRIC_FORMAT.format(iig), (int)x, (int) y);
        y+=this.INTERLINE_LEGEND;
        
        
    }    

    /**
     * This method is called from within the constructor to initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is always
     * regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        setDefaultCloseOperation(javax.swing.WindowConstants.EXIT_ON_CLOSE);
        setTitle("Sociogram");

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 710, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 447, Short.MAX_VALUE)
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        /* Set the Nimbus look and feel */
        //<editor-fold defaultstate="collapsed" desc=" Look and feel setting code (optional) ">
        /* If Nimbus (introduced in Java SE 6) is not available, stay with the default look and feel.
         * For details see http://download.oracle.com/javase/tutorial/uiswing/lookandfeel/plaf.html 
         */
        try {
            for (javax.swing.UIManager.LookAndFeelInfo info : javax.swing.UIManager.getInstalledLookAndFeels()) {
                if ("Nimbus".equals(info.getName())) {
                    javax.swing.UIManager.setLookAndFeel(info.getClassName());
                    break;
                }
            }
        } catch (ClassNotFoundException ex) {
            java.util.logging.Logger.getLogger(SociogramGUI.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (InstantiationException ex) {
            java.util.logging.Logger.getLogger(SociogramGUI.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (IllegalAccessException ex) {
            java.util.logging.Logger.getLogger(SociogramGUI.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        } catch (javax.swing.UnsupportedLookAndFeelException ex) {
            java.util.logging.Logger.getLogger(SociogramGUI.class.getName()).log(java.util.logging.Level.SEVERE, null, ex);
        }
        //</editor-fold>

        /* Create and display the form */
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new SociogramGUI().setVisible(true);
            }
        });
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
}
