/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package trainingsoci;
import sociogram.*;
import java.util.*;
import trainingdeck.*;


/**
 * A class with examples of ABS trainings.
 * @author Iván García-Magariño
 */
public class TestTraining {
    /** Cases for the training **/
    private List<CaseSim> cases;
    /** Trainer for ABS-SOCI **/
    TrainerSim trainer;
    /** It containes the time when the training started. It is used for 
     * measuring the elapsed time. */
    private long startTime;
    /** It contains the ellapsed time of the last training **/
    private long elapsedTime;
    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        
        // Complex training examples        
        TestTraining test = new TestTraining();
        test.createCasesAppletonEtAl();
        //test.createCasesAnDoan();
        test.startStatistics();
        test.train();
        test.stopStatistics();
        test.testExecution();
        
        // Simple training example
        //simpleTrainingExample();
    }
    /** A basic example of training ABS-SOCI with ATABS. **/
    public static void simpleTrainingExample(){
        // Creation of the simulator and the trainer
        TrainableSim trainableSoci = new TrainableSoci();
        TrainerSim trainer = new TrainerSim(trainableSoci);
        // Add a case considering the cohesion (i.e. IAg) 
        List<CaseSim> cases = new LinkedList();
        String deckNameChoice="suitable";
        int [] indexes =new int[2];
        indexes[0]=IndividualAgent.intValue(AgentType.PARTICIPANT);
        indexes[1]=IndividualAgent.intValue(AgentType.PARTICIPANT);        
        int sizeDeckAffinity= IndividualAgent.getSizeDeckAffinity(); 
        HashMap<String, Object> input = new HashMap();
        input.put("quiet", new Integer(0));
        input.put("participant", new Integer(100));
        input.put("tangent", new Integer(0));
        input.put("joker", new Integer(0));
        input.put("obstructive", new Integer(0));
        input.put("occasional", new Integer(0));
        HashMap<String, Object> output = new HashMap();
        output.put("IAg", new Double(0.11));
        output.put("ICg", new Double(0.60));
        ObjectiveSim objective= new ObjectiveIAg();
        CaseSim caseSim=new CaseSim(deckNameChoice, indexes, 
                sizeDeckAffinity, input, output, objective);
        cases.add(caseSim);      
        // Add the same case but considering the coherence (i.e. ICg)
        deckNameChoice ="coherenceSelection";
        int sizeDeckCoherence=IndividualAgent.getSizeDeckCoherenceSelection(); 
        objective= new ObjectiveICg();
        caseSim=new CaseSim(deckNameChoice, indexes, 
                sizeDeckCoherence, input, output, objective);
        cases.add(caseSim); 
        // Train the system and print the results
        trainableSoci=trainer.train(cases);
        printResults(trainableSoci);
    }
    /** Auxiliar method for the simple case that prints the results **/
    public static void printResults(TrainableSim trainableSoci){
        // Show the results
        System.out.println("Suitable Matrix:");
        int[] indexes={1,1};
        for(int i=0; i<AgentType.values().length; i++){
            for(int j=0; j<AgentType.values().length; j++){
                indexes[0]=i;
                indexes[1]=j;
                System.out.print(trainableSoci.getDeckValue("suitable", indexes)+"\t");
            }
            System.out.println();
        }
        System.out.println("CoherenceSelection="+trainableSoci.getDeckValue(
                "coherenceSelection", indexes));
    }
    
    /** Constructor **/
    public TestTraining(){
        cases = new LinkedList();
        TrainableSim trainableSoci = new TrainableSoci();
        trainer = new TrainerSim(trainableSoci);
        trainer.setMaxIterations(10);
    }
    
    /** It creates cases for guiding the training in the domain of 
     * (An & Doan, 2015)**/
    public void createCasesAnDoan(){
        // Common Variables
        CaseSim caseSim;
        HashMap<String, Object> input, output;
        int [] indexes =new int[2];
        String deckNameChoice;
        ObjectiveSim objective;
        int sizeDeckAffinity= IndividualAgent.getSizeDeckAffinity(); 
        int sizeDeckCoherence=IndividualAgent.getSizeDeckCoherenceSelection(); 
        
         // Case School 3
        deckNameChoice="suitable";
        indexes[0]=IndividualAgent.intValue(AgentType.QUIET);
        indexes[1]=IndividualAgent.intValue(AgentType.QUIET);        
        input = new HashMap();
        output = new HashMap();
        input = this.createInput(740, 47, 0, 0, 0, 0);
        output.put("IAg", new Double(0.00826));
        output.put("ICg", new Double(0.71607));
        objective= new ObjectiveIAg();
        caseSim=new CaseSim(deckNameChoice, indexes, 
                sizeDeckAffinity, input, output, objective);
        cases.add(caseSim); 
        // Same Case with ICg
        deckNameChoice ="coherenceSelection";
        objective= new ObjectiveICg();
        caseSim=new CaseSim(deckNameChoice, indexes, 
                sizeDeckCoherence, input, output, objective);
        cases.add(caseSim);  
       // School 4 
        deckNameChoice="suitable";
        indexes[0]=IndividualAgent.intValue(AgentType.QUIET);
        indexes[1]=IndividualAgent.intValue(AgentType.PARTICIPANT);   
        input = new HashMap<String, Object>();
        output = new HashMap<String, Object>();
        input = this.createInput(552, 38, 0, 0, 0, 0);
        output.put("IAg", new Double(0.00962));
        output.put("ICg", new Double(0.71353));
        objective= new ObjectiveIAg();
        caseSim=new CaseSim(deckNameChoice, indexes, 
                sizeDeckAffinity, input, output, objective);
        cases.add(caseSim);        
        indexes[0]=IndividualAgent.intValue(AgentType.PARTICIPANT);
        indexes[1]=IndividualAgent.intValue(AgentType.PARTICIPANT); 
        caseSim=new CaseSim(deckNameChoice, indexes, 
                sizeDeckAffinity, input, output, objective);
        cases.add(caseSim);        
    }
    /** It creates cases for guiding the training in the domain of  
     * (Appleton et al., 2013)**/
    public void createCasesAppletonEtAl(){
        // Common Variables
        CaseSim caseSim;
        HashMap<String, Object> input, output;
        int [] indexes =new int[2];
        String deckNameChoice;
        ObjectiveSim objective;
        int sizeDeckAffinity= IndividualAgent.getSizeDeckAffinity(); 
        int sizeDeckCoherence=IndividualAgent.getSizeDeckCoherenceSelection(); 
        deckNameChoice="suitable";
        objective= new ObjectiveIAg();
        
         // Case 
        indexes[0]=IndividualAgent.intValue(AgentType.PARTICIPANT);
        indexes[1]=IndividualAgent.intValue(AgentType.PARTICIPANT);        
        input = this.createInput(0, 100, 0, 0, 0, 0);
        output = new HashMap();
        output.put("IAg", new Double(0.45));
        caseSim=new CaseSim(deckNameChoice, indexes, sizeDeckAffinity, input, output, objective);
        cases.add(caseSim); 
        // Case 
        indexes[0]=IndividualAgent.intValue(AgentType.QUIET);
        indexes[1]=IndividualAgent.intValue(AgentType.QUIET);        
        input = this.createInput(100, 100, 0, 0, 0, 0);
        output = new HashMap();
        output.put("IAg", new Double(0.29));
        cases.add(new CaseSim(deckNameChoice, indexes, sizeDeckAffinity, input, output, objective));
        indexes[1]=IndividualAgent.intValue(AgentType.PARTICIPANT);        
        cases.add(new CaseSim(deckNameChoice, indexes, sizeDeckAffinity, input, output, objective));
        
        // Case 
        indexes[0]=IndividualAgent.intValue(AgentType.TANGENT);
        indexes[1]=IndividualAgent.intValue(AgentType.TANGENT);        
        input = this.createInput(100, 100, 100, 0, 0, 0);
        output = new HashMap();
        output.put("IAg", new Double(0.19));
        cases.add(new CaseSim(deckNameChoice, indexes, sizeDeckAffinity, input, output, objective));
        indexes[1]=IndividualAgent.intValue(AgentType.PARTICIPANT);        
        cases.add(new CaseSim(deckNameChoice, indexes, sizeDeckAffinity, input, output, objective)); 
        indexes[1]=IndividualAgent.intValue(AgentType.QUIET);        
        cases.add(new CaseSim(deckNameChoice, indexes, sizeDeckAffinity, input, output, objective)); 
        // Case 
        indexes[0]=IndividualAgent.intValue(AgentType.JOKER);
        indexes[1]=IndividualAgent.intValue(AgentType.JOKER);        
        input = this.createInput(100, 100, 100, 100, 0, 0);
        output = new HashMap();
        output.put("IAg", new Double(0.12));
        cases.add(new CaseSim(deckNameChoice, indexes, sizeDeckAffinity, input, output, objective));
        indexes[1]=IndividualAgent.intValue(AgentType.PARTICIPANT);        
        cases.add(new CaseSim(deckNameChoice, indexes, sizeDeckAffinity, input, output, objective)); 
        indexes[1]=IndividualAgent.intValue(AgentType.QUIET);        
        cases.add(new CaseSim(deckNameChoice, indexes, sizeDeckAffinity, input, output, objective)); 
        indexes[1]=IndividualAgent.intValue(AgentType.TANGENT);        
        cases.add(new CaseSim(deckNameChoice, indexes, sizeDeckAffinity, input, output, objective)); 
        // Case 
        indexes[0]=IndividualAgent.intValue(AgentType.OBSTRUCTIVE);
        indexes[1]=IndividualAgent.intValue(AgentType.OBSTRUCTIVE);        
        input = this.createInput(100, 100, 100, 100, 100, 0);
        output = new HashMap();
        output.put("IAg", new Double(0.20));
        cases.add(new CaseSim(deckNameChoice, indexes, sizeDeckAffinity, input, output, objective));
        indexes[1]=IndividualAgent.intValue(AgentType.PARTICIPANT);        
        cases.add(new CaseSim(deckNameChoice, indexes, sizeDeckAffinity, input, output, objective)); 
        indexes[1]=IndividualAgent.intValue(AgentType.QUIET);        
        cases.add(new CaseSim(deckNameChoice, indexes, sizeDeckAffinity, input, output, objective)); 
        indexes[1]=IndividualAgent.intValue(AgentType.TANGENT);        
        cases.add(new CaseSim(deckNameChoice, indexes, sizeDeckAffinity, input, output, objective)); 
        indexes[1]=IndividualAgent.intValue(AgentType.JOKER);        
        cases.add(new CaseSim(deckNameChoice, indexes, sizeDeckAffinity, input, output, objective)); 
         // Case 
        indexes[0]=IndividualAgent.intValue(AgentType.OCCASIONAL);
        indexes[1]=IndividualAgent.intValue(AgentType.OCCASIONAL);        
        input = this.createInput(100, 100, 100, 100, 100, 100);
        output = new HashMap();
        output.put("IAg", new Double(0.22));
        cases.add(new CaseSim(deckNameChoice, indexes, sizeDeckAffinity, input, output, objective));
        indexes[1]=IndividualAgent.intValue(AgentType.PARTICIPANT);        
        cases.add(new CaseSim(deckNameChoice, indexes, sizeDeckAffinity, input, output, objective)); 
        indexes[1]=IndividualAgent.intValue(AgentType.QUIET);        
        cases.add(new CaseSim(deckNameChoice, indexes, sizeDeckAffinity, input, output, objective)); 
        indexes[1]=IndividualAgent.intValue(AgentType.TANGENT);        
        cases.add(new CaseSim(deckNameChoice, indexes, sizeDeckAffinity, input, output, objective)); 
        indexes[1]=IndividualAgent.intValue(AgentType.JOKER);        
        cases.add(new CaseSim(deckNameChoice, indexes, sizeDeckAffinity, input, output, objective)); 
        indexes[1]=IndividualAgent.intValue(AgentType.OBSTRUCTIVE);        
        cases.add(new CaseSim(deckNameChoice, indexes, sizeDeckAffinity, input, output, objective));
        
        // Change the trainable max of iterations
        
    }
    /** It trains all the cases, and prints the results **/
    public void train(){
        TrainableSim trainableSoci = trainer.train(cases);
        // Show the results
        System.out.println("Suitable Matrix:");
        int[] indexes={1,1};
        for(int i=0; i<AgentType.values().length; i++){
            for(int j=0; j<AgentType.values().length; j++){
                indexes[0]=i;
                indexes[1]=j;
                System.out.print(trainableSoci.getDeckValue("suitable", indexes)+"\t");
            }
            System.out.println();
        }
        System.out.println("CoherenceSelection="+trainableSoci.getDeckValue(
                "coherenceSelection", indexes));
    }
    
    /** Creates an input of ABS-SOCI in a friendly way.
     @return An input for a training case. **/
    public HashMap<String, Object> createInput(int quite,
            int participant, int tangent,int joker, 
            int obstructive, int occasional){
        HashMap<String, Object> input;
        input = new HashMap<String, Object>();
        input.put("quiet", new Integer(quite));
        input.put("participant", new Integer(participant));
        input.put("tangent", new Integer(tangent));
        input.put("joker", new Integer(joker));
        input.put("obstructive", new Integer(obstructive));
        input.put("occasional", new Integer(occasional));
        return input;
    }
      
    /** It starts the statistics such as the measurement of elapsed time.
     */
    private void startStatistics(){
        // Starts the chronometer for measuring the elapsed time
        startTime = System.currentTimeMillis();        
    }
    /** It stops the chronometer and other statistics and show these **/
    private void stopStatistics(){
        // Get the elapsed time
        elapsedTime = System.currentTimeMillis() - startTime;
        // Get the memmory usage
        Runtime runtime = Runtime.getRuntime(); // Get the Java runtime
        runtime.gc(); // Run the garbage collector
        // Calculate the used memory in bytes
        double memoryBytes = runtime.totalMemory() - runtime.freeMemory();
        double memoryMB= memoryBytes/(1024*1024);
        System.out.println("Training obtained in "+
                elapsedTime+"ms, with " +memoryMB+" MB.");
    }
    
    /** Creates some cases for training the ABS-SOCI simulator. 
     @deprecated 
     **/
    public void createCases(){
        // Common Variables
        CaseSim caseSim;
        HashMap<String, Object> input, output;
        int [] indexes =new int[2];
        String deckNameChoice;
        ObjectiveSim objective;
        int sizeDeckAffinity= IndividualAgent.getSizeDeckAffinity(); 
        int sizeDeckCoherence=IndividualAgent.getSizeDeckCoherenceSelection(); 
        
         // Case with several objectives
        deckNameChoice="suitable";
        indexes[0]=IndividualAgent.intValue(AgentType.PARTICIPANT);
        indexes[1]=IndividualAgent.intValue(AgentType.PARTICIPANT);        
        input = new HashMap();
        output = new HashMap();
        input.put("quiet", new Integer(0));
        input.put("participant", new Integer(100));
        input.put("tangent", new Integer(0));
        input.put("joker", new Integer(0));
        input.put("obstructive", new Integer(0));
        input.put("occasional", new Integer(0));
        output.put("IAg", new Double(0.11));
        output.put("ICg", new Double(0.60));
        objective= new ObjectiveIAg();
        caseSim=new CaseSim(deckNameChoice, indexes, 
                sizeDeckAffinity, input, output, objective);
        cases.add(caseSim);      
        // Same Case with ICg
        deckNameChoice ="coherenceSelection";
        objective= new ObjectiveICg();
        caseSim=new CaseSim(deckNameChoice, indexes, 
                sizeDeckCoherence, input, output, objective);
        cases.add(caseSim);  
        // Case 
        deckNameChoice="suitable";
        indexes[0]=IndividualAgent.intValue(AgentType.QUIET);
        indexes[1]=IndividualAgent.intValue(AgentType.PARTICIPANT);        
        input = new HashMap<String, Object>();
        output = new HashMap<String, Object>();
        input.put("quiet", new Integer(100));
        input.put("participant", new Integer(10));
        input.put("tangent", new Integer(0));
        input.put("joker", new Integer(0));
        input.put("obstructive", new Integer(0));
        input.put("occasional", new Integer(0));
        output.put("IAg", new Double(0.03));
        output.put("ICg", new Double(0.60));
        objective= new ObjectiveIAg();
        caseSim=new CaseSim(deckNameChoice, indexes, 
                sizeDeckAffinity, input, output, objective);
        cases.add(caseSim);      
        // Simetric case: not necessary since the simetric is already considered
        // in the training
        /*indexes[0]=IndividualAgent.intValue(AgentType.PARTICIPANT);
        indexes[1]=IndividualAgent.intValue(AgentType.QUIET); 
        caseSim=new CaseSim(deckNameChoice, indexes, 
                sizeDeckAffinity, input, output, objective);
        cases.add(caseSim); */   
       // Coherence case
        deckNameChoice ="coherenceSelection";
        objective= new ObjectiveICg();
        caseSim=new CaseSim(deckNameChoice, indexes, 
                sizeDeckCoherence, input, output, objective);
        cases.add(caseSim);       
    }
    /** Some basic test of execution. 
     @deprecated **/
    public void testExecution(){
         HashMap<String, Object> input, output;
         
         // Set Configuration
        TrainableSoci trainableSoci = new TrainableSoci();
        int [] indexes={1,1};
        
        int counter=1;
        Iterator<CaseSim> it = cases.iterator();
        while(it.hasNext()){
            CaseSim c = it.next();
            output=trainableSoci.runSimulation(c.getInput());
            System.out.println("Case "+counter);
            System.out.println("IAg="+((Double)output.get("IAg")).doubleValue());
            System.out.println("ICg="+((Double)output.get("ICg")).doubleValue());
            System.out.println("IDg="+((Double)output.get("IDg")).doubleValue());
            System.out.println("IIg="+((Double)output.get("IIg")).doubleValue());
            counter++;
        }       
    }
    /** Just another example of training. 
     @deprecated **/
    public static void testTraining(){
        // Common Variables
        HashMap<String, Object> input, output;
        int [] indexes =new int[2];
        // Training  Case
        input = new HashMap<String, Object>();
        output = new HashMap<String, Object>();
        input.put("quite", new Integer(0));
        input.put("participant", new Integer(100));
        input.put("tangent", new Integer(0));
        input.put("joker", new Integer(0));
        input.put("obstructive", new Integer(0));
        input.put("occasional", new Integer(0));
        output.put("IAg", new Double(0.10));
        String deckNameChoice="suitable";
        indexes[0]=IndividualAgent.intValue(AgentType.PARTICIPANT);
        indexes[1]=IndividualAgent.intValue(AgentType.PARTICIPANT);        
        int sizeDeckAffinity=100; // Falta hacer un get del size en IndividualAgent
        ObjectiveSim objective= new ObjectiveIAg();
        // Training
        TrainableSoci trainableSoci = new TrainableSoci();
        TrainerSim trainer = new TrainerSim(trainableSoci);
        int suitable = trainer.trainDeck(deckNameChoice, indexes, sizeDeckAffinity, input, output, objective);   
        System.out.println("Suitable("+indexes[0]+","+indexes[1]+")="+suitable);
    }
}
